"""Return messages handler."""
# Third-Party Libraries
import click


def info_msg(text):
    """Informational message."""
    click.echo(click.style(text, fg="cyan"))


def success_msg(text):
    """Success message."""
    click.echo(click.style(text, fg="bright_green"))


def success_msg_via_pager(text):
    """Success message."""
    click.echo_via_pager(click.style(text, fg="green"))


def warning_msg(text):
    """Warning message."""
    click.echo(click.style(text, fg="yellow"))


def error_msg(text):
    """Error message."""
    click.echo(click.style(text, fg="red"))


def unknown_command():
    """Unknown command response."""
    click.echo("Sorry we didn't understand that command.")



#!/usr/bin/env python3

# Copyright 2023 Battelle Energy Alliance, LLC
import os
import contextlib
from functools import wraps
from time import monotonic

from tqdm import tqdm

from navv.message_handler import info_msg, success_msg, error_msg
from navv.validators import is_mac_address


@contextlib.contextmanager
def pushd(new_dir):
    previous_dir = os.getcwd()
    if not os.path.isdir(new_dir):
        os.makedirs(new_dir)
    os.chdir(new_dir)
    try:
        yield
    finally:
        os.chdir(previous_dir)


def timeit(func):
    @wraps(func)
    def _timeit(*args, **kwargs):
        start = monotonic()
        try:
            info_msg(f"running {func.__name__}")
            return func(*args, **kwargs)
        finally:
            end = monotonic()
            success_msg(f"{func.__name__} execution time:\n{end - start:0.2f} seconds")

    return _timeit


def trim_dns_data(data):
    """Find entries in dns log that contain no_error and return a dict of {ip: hostname,}"""
    ret_data = {}
    # log_list = log_to_list(data)
    info_msg("Trimming DNS.log data:")
    for row in tqdm(data.decode("utf-8 ").split("\n")[:-1]):
        # line_data = list(stream(row, '\t'))
        line_data = row.split("\t")
        if line_data[2] == "1" and line_data[3] == "NOERROR":
            for split in line_data[1].split(","):
                ret_data[split] = line_data[0]
    return ret_data


def get_mac_vendor(mac_vendors: dict, mac_address: str) -> str:
    """Return the vendor of the MAC address."""
    mac_address = mac_address.upper()

    if not is_mac_address(mac_address):
        error_msg(f"Invalid MAC address: {mac_address}")
        return f"Bad MAC address {mac_address}"

    try:
        vendor = [
            vendor["vendorName"]
            for vendor in mac_vendors
            if mac_address.startswith(vendor["macPrefix"])
        ][0]
    except IndexError:
        error_msg(f"Unknown vendor for MAC address: {mac_address}")
        return "Unknown Vendor"

    return vendor



import io
import os
import shutil
from tempfile import NamedTemporaryFile
from zipfile import ZipFile

import openpyxl

from navv.bll import get_inventory_report_df, get_snmp_df, get_zeek_df
from navv.spreadsheet_tools import (
    auto_adjust_width,
    create_analysis_array,
    get_inventory_data,
    get_package_data,
    get_segments_data,
    get_workbook,
    perform_analysis,
    write_conn_states_sheet,
    write_externals_sheet,
    write_inventory_report_sheet,
    write_snmp_sheet,
    write_stats_sheet,
    write_unknown_internals_sheet,
)
from navv.zeek import (
    get_conn_data,
    get_dns_data,
    get_snmp_data,
    run_zeek,
    perform_zeekcut,
)
from navv.utilities import pushd


def generate(customer_name, output_dir, pcap, zeek_logs_zip, spreadsheet):
    """Generate excel sheet."""
    with pushd(output_dir):
        pass

    if spreadsheet and spreadsheet.filename:
        wb = openpyxl.load_workbook(os.path.join(output_dir, spreadsheet.filename))
    else:
        file_name = os.path.join(output_dir, customer_name + "_network_analysis.xlsx")
        wb = get_workbook(file_name)

    # Extract Zeek logs from zip file
    if zeek_logs_zip:
        with ZipFile(f"{output_dir}/{zeek_logs_zip}", "r") as zip_file:
            zip_file.extractall(path=output_dir)
            zeek_logs = os.path.join(output_dir, zeek_logs_zip[:-4])
            os.remove(os.path.join(output_dir, zeek_logs_zip))
    else:
        zeek_logs = os.path.join(output_dir, "logs")

    services, conn_states = get_package_data()
    timer_data = dict()
    segments = get_segments_data(wb["Segments"])
    inventory = get_inventory_data(wb["Inventory Input"])

    if pcap and pcap.filename:
        run_zeek(os.path.join(output_dir, pcap.filename), zeek_logs, timer=timer_data)
    else:
        timer_data["run_zeek"] = "NOT RAN"

    # Get zeek data from conn.log, dns.log and snmp.log
    zeek_data = get_conn_data(zeek_logs)
    snmp_data = get_snmp_data(zeek_logs)
    dns_filtered = get_dns_data(customer_name, output_dir, zeek_logs)

    # Get dns data for resolution
    json_path = os.path.join(output_dir, f"{customer_name}_dns_data.json")

    # Get zeek dataframes
    zeek_df = get_zeek_df(zeek_data, dns_filtered)
    snmp_df = get_snmp_df(snmp_data)

    # Get inventory report dataframe
    inventory_df = get_inventory_report_df(zeek_df)

    # Turn zeekcut data into rows for spreadsheet
    rows = create_analysis_array(zeek_data, timer=timer_data)

    ext_IPs = set()
    unk_int_IPs = set()
    perform_analysis(
        wb,
        rows,
        services,
        conn_states,
        inventory,
        segments,
        dns_filtered,
        json_path,
        ext_IPs,
        unk_int_IPs,
        timer=timer_data,
    )

    write_inventory_report_sheet(inventory_df, wb)

    write_externals_sheet(ext_IPs, wb)

    write_unknown_internals_sheet(unk_int_IPs, wb)

    write_snmp_sheet(snmp_df, wb)

    auto_adjust_width(wb["Analysis"])
    times = (
        perform_zeekcut(fields=["ts"], log_file=os.path.join(zeek_logs, "conn.log"))
        .decode("utf-8")
        .split("\n")[:-1]
    )
    forward = sorted(times)
    start = float(forward[0])
    end = float(forward[len(forward) - 1])
    cap_time = end - start
    timer_data[
        "Length of Capture time"
    ] = "{} day(s) {} hour(s) {} minutes {} seconds".format(
        int(cap_time / 86400),
        int(cap_time % 86400 / 3600),
        int(cap_time % 3600 / 60),
        int(cap_time % 60),
    )
    write_stats_sheet(wb, timer_data)
    write_conn_states_sheet(conn_states, wb)

    memfile: io.BytesIO
    with NamedTemporaryFile() as tmp:
        wb.save(tmp.name)
        tmp.seek(0)
        memfile = io.BytesIO(tmp.read())

    shutil.rmtree(output_dir)
    return memfile



from ipaddress import IPv4Address, IPv6Address
import re


def is_ipv4_address(ip_address: str) -> bool:
    """Return True if address is a valid IPv4 address."""
    try:
        IPv4Address(ip_address)
        return True
    except ValueError:
        return False


def is_ipv6_address(ip_address: str) -> bool:
    """Return True if address is a valid IPv6 address."""
    try:
        IPv6Address(ip_address)
        return True
    except ValueError:
        return False


def is_mac_address(mac_address: str) -> bool:
    """Return True if address is a valid MAC address."""
    if re.match(
        "[0-9a-f]{2}([-:])[0-9a-f]{2}(\\1[0-9a-f]{2}){4}$", mac_address.lower()
    ):
        return True
    return False
